@@ -1,9 +1,14 @@
 /*
-   (c) Copyright 2000  convergence integrated media GmbH.
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
    All rights reserved.
 
-   Written by Denis Oliver Kropp <dok@convergence.de> and
-              Andreas Hundt <andi@convergence.de>.
+   Written by Denis Oliver Kropp <dok@directfb.org>,
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -21,7 +26,9 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <asm/types.h>
+#include <config.h>
+
+#include <dfb_types.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -31,17 +38,28 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
+#include <fbdev/fb.h>
+
 #include <directfb.h>
 
+#include <direct/messages.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
+#include <core/screens.h>
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/convert.h>
 
+#include <core/graphics_driver.h>
+
+
+DFB_GRAPHICS_DRIVER( ati128 )
+
+
 #include "regs.h"
 #include "mmio.h"
 #include "ati128_state.h"
@@ -70,30 +88,41 @@
 
 
 /* macro for S12.0 and S14.0 format */
-#define S12(val) (((__u16)((__s16)(val)))&0x3fff)
-#define S14(val) (((__u16)((__s16)(val)))&0x3fff)
+#define S12(val) (((u16)((s16)(val)))&0x3fff)
+#define S14(val) (((u16)((s16)(val)))&0x3fff)
 
 /** CARD FUNCTIONS **/
-static void ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect );
-static void ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect );
-static void ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect );
-static void ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect );
+static bool ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect );
+static bool ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect );
+static bool ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect );
+static bool ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect );
 
 /* required implementations */
 
-static void ati128EngineSync( void *drv, void *dev )
+static DFBResult ati128EngineSync( void *drv, void *dev )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
 
      ati128_waitidle( adrv, adev );
+
+     return DFB_OK;
+}
+
+static bool ati128_check_blend( CardState *state )
+{
+   if (state->dst_blend == DSBF_SRCALPHASAT)
+      return false;
+
+   return true;
 }
 
 static void ati128CheckState( void *drv, void *dev,
                               CardState *state, DFBAccelerationMask accel )
 {
-     switch (state->destination->format) {
-          case DSPF_RGB15:
+     switch (state->destination->config.format) {
+          case DSPF_RGB332:
+          case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB24:
           case DSPF_RGB32:
@@ -110,19 +139,29 @@ static void ati128CheckState( void *drv, void *dev,
 
      /* if there are no other drawing flags than the supported */
      if (!(accel & ~ATI128_SUPPORTED_DRAWINGFUNCTIONS) &&
-         !(state->drawingflags & ~ATI128_SUPPORTED_DRAWINGFLAGS))
+         !(state->drawingflags & ~ATI128_SUPPORTED_DRAWINGFLAGS)) {
+          if (state->drawingflags & DSDRAW_BLEND &&
+              !ati128_check_blend( state ))
+               return;
+
           state->accel |= ATI128_SUPPORTED_DRAWINGFUNCTIONS;
+     }
 
      /* if there are no other blitting flags than the supported
         and the source has the minimum size */
      if (!(accel & ~ATI128_SUPPORTED_BLITTINGFUNCTIONS) &&
          !(state->blittingflags & ~ATI128_SUPPORTED_BLITTINGFLAGS)  &&
          state->source &&
-         state->source->width  >= 8 &&
-         state->source->height >= 8 )
+         state->source->config.size.w >= 8 &&
+         state->source->config.size.h >= 8 )
      {
-          switch (state->source->format) {
-               case DSPF_RGB15:
+          if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL &&
+              !ati128_check_blend( state ))
+               return;
+
+          switch (state->source->config.format) {
+               case DSPF_RGB332:
+               case DSPF_ARGB1555:
                case DSPF_RGB16:
                case DSPF_RGB24:
                case DSPF_RGB32:
@@ -142,65 +181,95 @@ static void ati128SetState( void *drv, void *dev,
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
 
-     if ((state->modified & SMF_SOURCE)  &&  state->source)
-          ati128_set_source( adrv, adev, state );
+     if (state->mod_hw & SMF_SOURCE)
+         adev->v_source = 0;
 
-     if (state->modified & SMF_DESTINATION)
-          ati128_set_destination( adrv, adev, state );
+     if (state->mod_hw & SMF_DESTINATION)
+          adev->v_destination = adev->v_color = 0;
 
-     if (state->modified & SMF_CLIP)
-          ati128_set_clip( adrv, adev, state );
+     if (state->mod_hw & SMF_COLOR)
+          adev->v_color = 0;
 
-     if (state->modified & SMF_COLOR)
-          ati128_set_color( adrv, adev, state );
+     if (state->mod_hw & SMF_SRC_COLORKEY)
+          adev->v_src_colorkey = 0;
 
-     if (state->modified & SMF_SRC_COLORKEY)
-          ati128_out32( adrv->mmio_base, CLR_CMP_CLR_SRC, state->src_colorkey );
+     if (state->mod_hw & SMF_BLITTING_FLAGS)
+          adev->v_blittingflags = 0;
 
-     if (state->modified & SMF_BLITTING_FLAGS)
-          ati128_set_blittingflags( adrv, adev, state );
+     if (state->mod_hw & (SMF_SRC_BLEND | SMF_DST_BLEND))
+          adev->v_blending_function = 0;
 
-     if (state->modified & (SMF_SRC_BLEND | SMF_DST_BLEND))
-          ati128_set_blending_function( adrv, adev, state );
+     ati128_set_destination( adrv, adev, state);
 
-     if (state->modified & SMF_DRAWING_FLAGS) {
-          if (state->drawingflags & DSDRAW_BLEND) {
-               funcs->FillRectangle = ati128FillBlendRectangle;
-               funcs->DrawRectangle = ati128DrawBlendRectangle;
-          }
-          else {
-               funcs->FillRectangle = ati128FillRectangle;
-               funcs->DrawRectangle = ati128DrawRectangle;
-          }
+     switch (accel) {
+          case DFXL_FILLRECTANGLE:
+          case DFXL_DRAWRECTANGLE:
+               if (state->drawingflags & DSDRAW_BLEND) {
+                    ati128_set_blending_function( adrv, adev, state );
+                    funcs->FillRectangle = ati128FillBlendRectangle;
+                    funcs->DrawRectangle = ati128DrawBlendRectangle;
+               }
+               else {
+                    funcs->FillRectangle = ati128FillRectangle;
+                    funcs->DrawRectangle = ati128DrawRectangle;
+               }
+          case DFXL_DRAWLINE:
+               ati128_set_color( adrv, adev, state );
+               state->set |= DFXL_FILLRECTANGLE | DFXL_DRAWLINE | DFXL_DRAWRECTANGLE ;
+               break;
+
+          case DFXL_BLIT:
+          case DFXL_STRETCHBLIT:
+               ati128_set_source( adrv, adev, state );
+               if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
+                    ati128_set_blending_function( adrv, adev, state );
+               if (state->blittingflags & DSBLIT_SRC_COLORKEY)
+                    ati128_set_src_colorkey( adrv, adev, state );
+               ati128_set_blittingflags( adrv, adev, state );
+               state->set |= DFXL_BLIT | DFXL_STRETCHBLIT;
+               break;
+
+          default:
+               D_BUG( "unexpected drawing/blitting function" );
+               break;
      }
+
+     if (state->mod_hw & SMF_CLIP)
+          ati128_set_clip( adrv, adev, state);
+
+     state->mod_hw = 0;
 }
 
-static void ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect )
+static bool ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     ati128_waitfifo( adrv, adev, 4 );
+     ati128_waitfifo( adrv, adev, 5 );
      /* set the destination datatype */
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | BRUSH_SOLIDCOLOR );
+     /* set direction */
+     ati128_out32( mmio, DP_CNTL, DST_X_LEFT_TO_RIGHT | DST_Y_TOP_TO_BOTTOM);
      /* set the drawing command */
      ati128_out32( mmio, DP_MIX, ROP3_PATCOPY | DP_SRC_RECT );
      /* set parameters */
      ati128_out32( mmio, DST_Y_X, (S14(rect->y) << 16) | S12(rect->x) );
      /* this executes the drawing command */
      ati128_out32( mmio, DST_HEIGHT_WIDTH, (rect->h << 16) | rect->w );
+
+     return true;
 }
 
-static void ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
+static bool ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
+     u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
      ati128_waitidle( adrv, adev );
-     *((__u32*)  gfxcard_memory_virtual(fts)  ) = adev->fake_texture_color;
+     *((u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
      ati128_waitidle( adrv, adev );
 
      ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
@@ -229,17 +298,21 @@ static void ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
      ati128_out32( mmio, SCALE_3D_CNTL, 0x00000000 );
      ati128_out32( mmio, TEX_CNTL,  0);
      adev->fake_texture_number++;
+
+     return true;
 }
 
-static void ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
+static bool ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     ati128_waitfifo( adrv, adev, 2 );
+     ati128_waitfifo( adrv, adev, 3 );
      /* set the destination datatype */
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | BRUSH_SOLIDCOLOR );
+     /* set direction */
+     ati128_out32( mmio, DP_CNTL, DST_X_LEFT_TO_RIGHT | DST_Y_TOP_TO_BOTTOM);
      /* set the drawing command */
      ati128_out32( mmio, DP_MIX, ROP3_PATCOPY | DP_SRC_RECT );
 
@@ -255,18 +328,20 @@ static void ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
      /* fourth line */
      ati128_out32( mmio, DST_Y_X, (S14(rect->y) << 16) | S12(rect->x+rect->w-1));
      ati128_out32( mmio, DST_HEIGHT_WIDTH, rect->h << 16 | 1);
+
+     return true;
 }
 
-static void ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
+static bool ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
+     u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
 
      ati128_waitidle( adrv, adev );
-     *((__u32*)  gfxcard_memory_virtual(fts)  ) = adev->fake_texture_color;
+     *((u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
      ati128_waitidle( adrv, adev );
 
      ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
@@ -307,21 +382,23 @@ static void ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
      ati128_out32( mmio, SCALE_3D_CNTL, 0x00000000 );
      ati128_out32( mmio, TEX_CNTL, 0 );
      adev->fake_texture_number++;
+
+     return true;
 }
 
 
-static void ati128DrawLine( void *drv, void *dev, DFBRegion *line )
+static bool ati128DrawLine( void *drv, void *dev, DFBRegion *line )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      int dx, dy;
      int small, large;
      int x_dir, y_dir, y_major;
      int err, inc, dec;
 
-     // Determine x & y deltas and x & y direction bits.
+     /* Determine x & y deltas and x & y direction bits. */
      if (line->x1 < line->x2) {
           dx = line->x2 - line->x1;
           x_dir = 1 << 31;
@@ -339,7 +416,8 @@ static void ati128DrawLine( void *drv, void *dev, DFBRegion *line )
           dy = line->y1 - line->y2;
           y_dir = 0 << 15;
      }
-     // Determine x & y min and max values; also determine y major bit.
+
+     /* Determine x & y min and max values; also determine y major bit. */
      if (dx < dy) {
           small = dx;
           large = dy;
@@ -350,37 +428,42 @@ static void ati128DrawLine( void *drv, void *dev, DFBRegion *line )
           large = dx;
           y_major = 0 << 2;
      }
-     // Calculate Bresenham parameters and draw line.
+
+     /* Calculate Bresenham parameters and draw line. */
      err = -large;
      inc = small * 2;
      dec = large *(-2);
 
-     ati128_waitfifo( adrv, adev, 7 );
+     ati128_waitfifo( adrv, adev, 8 );
      /* set the destination datatype */
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | BRUSH_SOLIDCOLOR | ROP3_SRCCOPY );
+     ati128_out32( mmio, DP_MIX, ROP3_PATCOPY );
+
      /* set start coorinates */
      ati128_out32( mmio, DST_Y_X, (S14(line->y1) << 16) | S12(line->x1));
      /* allow setting of last pel bit and polygon
         outline bit for line drawing */
      ati128_out32( mmio, DP_CNTL_XDIR_YDIR_YMAJOR,
                    y_major | y_dir | x_dir );
-     // set bresenham registers and start drawing
+     /* set bresenham registers and start drawing */
      ati128_out32( mmio, DST_BRES_ERR, err );
      ati128_out32( mmio, DST_BRES_INC, inc );
      ati128_out32( mmio, DST_BRES_DEC, dec );
      ati128_out32( mmio, DST_BRES_LNTH, large + 1 );
+
+     return true;
 }
 
-static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
+static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 src = 0;
+     u32 src = 0;
 
-     __u32 scalex = (__u32)(((double)sr->w/(double)dr->w) * 65536);
-     __u32 scaley = (__u32)(((double)sr->h/(double)dr->h) * 65536);
+     u32 scalex = (u32)(((double)sr->w/(double)dr->w) * 65536);
+     u32 scaley = (u32)(((double)sr->h/(double)dr->h) * 65536);
 
      ati128_waitfifo( adrv, adev, 9 );
 
@@ -388,29 +471,42 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
 
      ati128_out32( mmio, CLR_CMP_CNTL, adev->ATI_color_compare );
 
-     switch (adev->source->format) {
-          case DSPF_RGB15:
-               ati128_out32( mmio, SCALE_3D_DATATYPE, DST_15BPP );
+     switch (adev->source->config.format) {
+          case DSPF_RGB332:
+               ati128_out32( mmio, SCALE_3D_DATATYPE, DST_8BPP_RGB332 );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>4);
+                             adev->src->pitch >>3);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*2;
+                     adev->src->pitch + sr->x;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
+               break;
+          case DSPF_ARGB1555: /* FIXME: alpha channel will be zero ;( */
+               ati128_out32( mmio, SCALE_3D_DATATYPE, DST_15BPP );
 
+               ati128_out32( mmio, SCALE_PITCH,
+                             adev->src->pitch >>4);
+
+               src = adev->src->offset +
+                     sr->y *
+                     adev->src->pitch + sr->x*2;
+
+               ati128_out32( mmio, TEX_CNTL, 0);
+
+               break;
           case DSPF_RGB16:
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_16BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>4);
+                             adev->src->pitch >>4);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*2;
+                     adev->src->pitch + sr->x*2;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -419,11 +515,11 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_24BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>3);
+                             adev->src->pitch >>3);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*3;
+                     adev->src->pitch + sr->x*3;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -432,11 +528,11 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>5);
+                             adev->src->pitch >>5);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*4;
+                     adev->src->pitch + sr->x*4;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -445,11 +541,11 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>5);
+                             adev->src->pitch >>5);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*4;
+                     adev->src->pitch + sr->x*4;
 
                if (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
                     ati128_out32( mmio, TEX_CNTL, TEX_CNTL_ALPHA_EN_ON );
@@ -457,13 +553,18 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                     ati128_out32( mmio, TEX_CNTL, 0 );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
-               return;
+               D_BUG( "unexpected pixelformat!" );
+               return false;
      }
 
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | SRC_DSTCOLOR );
+
      /* set the blend function */
-     ati128_out32( mmio, SCALE_3D_CNTL, adev->ATI_blend_function );
+     if (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
+          ati128_out32( mmio, SCALE_3D_CNTL, adev->ATI_blend_function );
+     else
+          ati128_out32( mmio, SCALE_3D_CNTL, SCALE_3D_CNTL_SCALE_3D_FN_SCALE );
+
      /* set up source data and copy type */
      ati128_out32( mmio, DP_MIX, ROP3_SRCCOPY | DP_SRC_RECT );
      /* set source offset */
@@ -489,23 +590,25 @@ static void ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
      /* set CLR_CMP_CNTL to zero, to insure that drawing funcions work corrently */
      if (adev->ATI_color_compare)
           ati128_out32( mmio, CLR_CMP_CNTL, 0 );
+
+     return true;
 }
 
-static void ati128Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
+static bool ati128Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 dir_cmd = 0;
+     u32 dir_cmd = 0;
 
-     if ((adev->source->format != adev->destination->format) ||
-         adev->blittingflags == DSBLIT_BLEND_ALPHACHANNEL)
+     if ((adev->source->config.format != adev->destination->config.format) ||
+         (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL))
      {
           DFBRectangle sr = { rect->x, rect->y, rect->w, rect->h };
           DFBRectangle dr = { dx, dy, rect->w, rect->h };
           ati128StretchBlit( adrv, adev, &sr, &dr );
-          return;
+          return true;
      }
 
      /* check which blitting direction should be used */
@@ -549,31 +652,26 @@ static void ati128Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy
           ati128_out32( mmio, DP_CNTL, DST_X_LEFT_TO_RIGHT |
                                        DST_Y_TOP_TO_BOTTOM );
      }
+
+     return true;
 }
 
 /* exported symbols */
 
-int
-driver_get_abi_version()
+static int
+driver_probe( CoreGraphicsDevice *device )
 {
-     return DFB_GRAPHICS_DRIVER_ABI_VERSION;
-}
-
-int
-driver_probe( GraphicsDevice *device )
-{
-#ifdef FB_ACCEL_ATI_RAGE128
-     switch (gfxcard_get_accelerator( device )) {
+     switch (dfb_gfxcard_get_accelerator( device )) {
           case FB_ACCEL_ATI_RAGE128:          /* ATI Rage 128 */
                return 1;
      }
-#endif
+
      return 0;
 }
 
 
-void
-driver_get_info( GraphicsDevice     *device,
+static void
+driver_get_info( CoreGraphicsDevice *device,
                  GraphicsDriverInfo *info )
 {
      /* fill driver info structure */
@@ -583,23 +681,25 @@ driver_get_info( GraphicsDevice     *device,
 
      snprintf( info->vendor,
                DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "directfb.org" );
 
      info->version.major = 0;
-     info->version.minor = 1;
+     info->version.minor = 2;
 
      info->driver_data_size = sizeof (ATI128DriverData);
      info->device_data_size = sizeof (ATI128DeviceData);
 }
 
-DFBResult
-driver_init_driver( GraphicsDevice      *device,
+static DFBResult
+driver_init_driver( CoreGraphicsDevice  *device,
                     GraphicsDeviceFuncs *funcs,
-                    void                *driver_data )
+                    void                *driver_data,
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
 
-     adrv->mmio_base = (volatile __u8*) gfxcard_map_mmio( device, 0, -1 );
+     adrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!adrv->mmio_base)
           return DFB_IO;
 
@@ -613,19 +713,23 @@ driver_init_driver( GraphicsDevice      *device,
      funcs->Blit          = ati128Blit;
      funcs->StretchBlit   = ati128StretchBlit;
 
+     /* overlay support */
+     dfb_layers_register( dfb_screens_at(DSCID_PRIMARY),
+                          driver_data, &atiOverlayFuncs );
+
      return DFB_OK;
 }
 
 
-DFBResult
-driver_init_device( GraphicsDevice     *device,
+static DFBResult
+driver_init_device( CoreGraphicsDevice *device,
                     GraphicsDeviceInfo *device_info,
                     void               *driver_data,
                     void               *device_data )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
      ATI128DeviceData *adev = (ATI128DeviceData*) device_data;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      /* fill device info */
      snprintf( device_info->name,
@@ -668,44 +772,39 @@ driver_init_device( GraphicsDevice     *device,
      ati128_out32( mmio, TEX_CNTL, 0x00000000 );
 
      /* reserve 32bit pixel for fake texture at end of framebuffer */
-     adev->ATI_fake_texture_src = gfxcard_reserve_memory( device, 4*32 );
+     adev->ATI_fake_texture_src = dfb_gfxcard_reserve_memory( device, 4*32 );
 
      return DFB_OK;
 }
 
-void
-driver_init_layers()
-{
-}
-
-void
-driver_close_device( GraphicsDevice *device,
-                     void           *driver_data,
-                     void           *device_data )
+static void
+driver_close_device( CoreGraphicsDevice *device,
+                     void               *driver_data,
+                     void               *device_data )
 {
      ATI128DeviceData *adev = (ATI128DeviceData*) device_data;
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     DEBUGMSG( "DirectFB/ATI128: FIFO Performance Monitoring:\n" );
-     DEBUGMSG( "DirectFB/ATI128:  %9d ati128_waitfifo calls\n",
+     D_DEBUG( "DirectFB/ATI128: FIFO Performance Monitoring:\n" );
+     D_DEBUG( "DirectFB/ATI128:  %9d ati128_waitfifo calls\n",
                adev->waitfifo_calls );
-     DEBUGMSG( "DirectFB/ATI128:  %9d register writes (ati128_waitfifo sum)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d register writes (ati128_waitfifo sum)\n",
                adev->waitfifo_sum );
-     DEBUGMSG( "DirectFB/ATI128:  %9d FIFO wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d FIFO wait cycles (depends on CPU)\n",
                adev->fifo_waitcycles );
-     DEBUGMSG( "DirectFB/ATI128:  %9d IDLE wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d IDLE wait cycles (depends on CPU)\n",
                adev->idle_waitcycles );
-     DEBUGMSG( "DirectFB/ATI128:  %9d FIFO space cache hits(depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d FIFO space cache hits(depends on CPU)\n",
                adev->fifo_cache_hits );
-     DEBUGMSG( "DirectFB/ATI128: Conclusion:\n" );
-     DEBUGMSG( "DirectFB/ATI128:  Average register writes/ati128_waitfifo"
+     D_DEBUG( "DirectFB/ATI128: Conclusion:\n" );
+     D_DEBUG( "DirectFB/ATI128:  Average register writes/ati128_waitfifo"
                "call:%.2f\n",
                adev->waitfifo_sum/(float)(adev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/ATI128:  Average wait cycles/ati128_waitfifo call:"
+     D_DEBUG( "DirectFB/ATI128:  Average wait cycles/ati128_waitfifo call:"
                " %.2f\n",
                adev->fifo_waitcycles/(float)(adev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/ATI128:  Average fifo space cache hits: %02d%%\n",
+     D_DEBUG( "DirectFB/ATI128:  Average fifo space cache hits: %02d%%\n",
                (int)(100 * adev->fifo_cache_hits/
                (float)(adev->waitfifo_calls)) );
 
@@ -733,12 +832,11 @@ driver_close_device( GraphicsDevice *device,
      ati128_out32( mmio, TEX_CNTL, 0x00000000 );
 }
 
-void
-driver_close_driver( GraphicsDevice *device,
-                     void           *driver_data )
+static void
+driver_close_driver( CoreGraphicsDevice *device,
+                     void               *driver_data )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
 
-     gfxcard_unmap_mmio( device, adrv->mmio_base, -1 );
+     dfb_gfxcard_unmap_mmio( device, adrv->mmio_base, -1 );
 }
-
