@@ -1,9 +1,14 @@
 /*
-   (c) Copyright 2000  convergence integrated media GmbH.
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
    All rights reserved.
 
-   Written by Denis Oliver Kropp <dok@convergence.de> and
-              Andreas Hundt <andi@convergence.de>.
+   Written by Denis Oliver Kropp <dok@directfb.org>,
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -21,7 +26,9 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <asm/types.h>
+#include <config.h>
+
+#include <dfb_types.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -30,28 +37,36 @@
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <malloc.h>
+
+#include <fbdev/fb.h>
 
 #include <directfb.h>
 
+#include <direct/messages.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/convert.h>
 #include <gfx/util.h>
 #include <misc/conf.h>
 
+#include <core/graphics_driver.h>
+
+
+DFB_GRAPHICS_DRIVER( tdfx )
+
 #include "tdfx.h"
 
-static void tdfxFillRectangle2D( void *drv, void *dev, DFBRectangle *rect );
-static void tdfxFillRectangle3D( void *drv, void *dev, DFBRectangle *rect );
-static void tdfxFillTriangle2D ( void *drv, void *dev, DFBTriangle  *tri );
-static void tdfxFillTriangle3D ( void *drv, void *dev, DFBTriangle  *tri );
-static void tdfxDrawLine2D     ( void *drv, void *dev, DFBRegion    *line );
+static bool tdfxFillRectangle2D( void *drv, void *dev, DFBRectangle *rect );
+static bool tdfxFillRectangle3D( void *drv, void *dev, DFBRectangle *rect );
+static bool tdfxFillTriangle2D ( void *drv, void *dev, DFBTriangle  *tri );
+static bool tdfxFillTriangle3D ( void *drv, void *dev, DFBTriangle  *tri );
+static bool tdfxDrawLine2D     ( void *drv, void *dev, DFBRegion    *line );
 //static void tdfxDrawLine3D     ( void *drv, void *dev, DFBRegion    *line );
 
 typedef struct {
@@ -76,7 +91,7 @@ typedef struct {
 } TDFXDeviceData;
 
 typedef struct {
-     volatile __u8 *mmio_base;
+     volatile u8   *mmio_base;
      Voodoo2D      *voodoo2D;
      Voodoo3D      *voodoo3D;
 } TDFXDriverData;
@@ -84,7 +99,8 @@ typedef struct {
 
 
 static inline void tdfx_waitfifo( TDFXDriverData *tdrv,
-                                  TDFXDeviceData *tdev, int space )
+                                  TDFXDeviceData *tdev,
+                                  unsigned int space )
 {
      int timeout = 1000000;
 
@@ -100,15 +116,14 @@ static inline void tdfx_waitfifo( TDFXDriverData *tdrv,
                     break;
 
           }
-     }
-     else {
+     } else {
           tdev->fifo_cache_hits++;
      }
 
      tdev->fifo_space -= space;
 
      if (!timeout)
-          CAUTION( "timeout during waitfifo!" );
+          D_WARN( "timeout during waitfifo!" );
 }
 
 static inline void tdfx_waitidle( TDFXDriverData *tdrv,
@@ -125,17 +140,17 @@ static inline void tdfx_waitidle( TDFXDriverData *tdrv,
           tdev->idle_waitcycles++;
 
           i = (tdrv->voodoo2D->status & (0xF << 7))  ?  0 : i + 1;
-          if(i == 3)
+          if (i == 3)
                return;
 
      }
 
-     BUG( "timeout during waitidle!\n");
+     D_BUG( "timeout during waitidle!\n");
 }
 
 
 static int blitFormat[] = {
-     2, /* DSPF_RGB15 */
+     2, /* DSPF_ARGB1555 */
      3, /* DSPF_RGB16 */
      4, /* DSPF_RGB24 */
      5, /* DSPF_RGB32 */
@@ -148,7 +163,6 @@ static inline void tdfx_validate_source2D( TDFXDriverData *tdrv,
                                            CardState      *state )
 {
      CoreSurface   *source   = state->source;
-     SurfaceBuffer *buffer   = source->front_buffer;
      Voodoo2D      *voodoo2D = tdrv->voodoo2D;
 
      if (tdev->v_source2D)
@@ -156,9 +170,9 @@ static inline void tdfx_validate_source2D( TDFXDriverData *tdrv,
 
      tdfx_waitfifo( tdrv, tdev, 2 );
 
-     voodoo2D->srcBaseAddr = buffer->video.offset & 0xFFFFFF;
-     voodoo2D->srcFormat   = (buffer->video.pitch & 0x3FFF) |
-                             (blitFormat[DFB_PIXELFORMAT_INDEX(source->format)] << 16);
+     voodoo2D->srcBaseAddr = state->src.offset & 0xFFFFFF;
+     voodoo2D->srcFormat   = (state->src.pitch & 0x3FFF) |
+                             (blitFormat[DFB_PIXELFORMAT_INDEX(source->config.format)] << 16);
 
      tdev->v_source2D = 1;
 }
@@ -168,7 +182,6 @@ static inline void tdfx_validate_destination2D( TDFXDriverData *tdrv,
                                                 CardState      *state )
 {
      CoreSurface   *destination = state->destination;
-     SurfaceBuffer *buffer      = destination->back_buffer;
      Voodoo2D      *voodoo2D    = tdrv->voodoo2D;
 
      if (tdev->v_destination2D)
@@ -176,9 +189,9 @@ static inline void tdfx_validate_destination2D( TDFXDriverData *tdrv,
 
      tdfx_waitfifo( tdrv, tdev, 2 );
 
-     voodoo2D->dstBaseAddr = buffer->video.offset;
-     voodoo2D->dstFormat   = (buffer->video.pitch & 0x3FFF) |
-                             (blitFormat[DFB_PIXELFORMAT_INDEX(destination->format)] << 16);
+     voodoo2D->dstBaseAddr = state->dst.offset;
+     voodoo2D->dstFormat   = (state->dst.pitch & 0x3FFF) |
+                             (blitFormat[DFB_PIXELFORMAT_INDEX(destination->config.format)] << 16);
 
      tdev->v_destination2D = 1;
 }
@@ -188,17 +201,16 @@ static inline void tdfx_validate_destination3D( TDFXDriverData *tdrv,
                                                 CardState      *state )
 {
      CoreSurface   *destination = state->destination;
-     SurfaceBuffer *buffer      = destination->back_buffer;
      Voodoo3D      *voodoo3D    = tdrv->voodoo3D;
 
-     __u32 lfbmode = TDFX_LFBMODE_PIXEL_PIPELINE_ENABLE;
-     __u32 fbzMode = (1 << 9) | 1;
+     u32 lfbmode = TDFX_LFBMODE_PIXEL_PIPELINE_ENABLE;
+     u32 fbzMode = (1 << 9) | 1;
 
      if (tdev->v_destination3D)
           return;
 
-     switch (destination->format) {
-          case DSPF_RGB15:
+     switch (destination->config.format) {
+          case DSPF_ARGB1555:
                lfbmode |= TDFX_LFBMODE_RGB555;
                break;
           case DSPF_RGB16:
@@ -212,7 +224,7 @@ static inline void tdfx_validate_destination3D( TDFXDriverData *tdrv,
                lfbmode |= TDFX_LFBMODE_ARGB8888;
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
@@ -220,8 +232,8 @@ static inline void tdfx_validate_destination3D( TDFXDriverData *tdrv,
 
      voodoo3D->lfbMode = lfbmode;
      voodoo3D->fbzMode = fbzMode;
-     voodoo3D->colBufferAddr = buffer->video.offset;
-     voodoo3D->colBufferStride = buffer->video.pitch;
+     voodoo3D->colBufferAddr = state->dst.offset;
+     voodoo3D->colBufferStride = state->dst.pitch;
 
      tdev->v_destination3D = 1;
 }
@@ -252,14 +264,15 @@ static inline void tdfx_validate_colorFore( TDFXDriverData *tdrv,
 
      tdfx_waitfifo( tdrv, tdev, 1 );
 
-     switch (state->destination->format) {
+     switch (state->destination->config.format) {
           case DSPF_A8:
                tdrv->voodoo2D->colorFore = state->color.a;
                break;
-          case DSPF_RGB15:
-               tdrv->voodoo2D->colorFore = PIXEL_RGB15( state->color.r,
-                                                        state->color.g,
-                                                        state->color.b );
+          case DSPF_ARGB1555:
+               tdrv->voodoo2D->colorFore = PIXEL_ARGB1555( state->color.a,
+                                                           state->color.r,
+                                                           state->color.g,
+                                                           state->color.b );
                break;
           case DSPF_RGB16:
                tdrv->voodoo2D->colorFore = PIXEL_RGB16( state->color.r,
@@ -279,7 +292,7 @@ static inline void tdfx_validate_colorFore( TDFXDriverData *tdrv,
                                                         state->color.b );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
@@ -311,10 +324,10 @@ static inline void tdfx_validate_alphaMode( TDFXDriverData *tdrv,
      tdfx_waitfifo( tdrv, tdev, 1 );
 
      tdrv->voodoo3D->alphaMode = TDFX_ALPHAMODE_BLEND_ENABLE |
-                                (tdfxBlendFactor[state->src_blend] <<  8) |
-                                (tdfxBlendFactor[state->src_blend] << 16) |
-                                (tdfxBlendFactor[state->dst_blend] << 12) |
-                                (tdfxBlendFactor[state->dst_blend] << 20);
+                                 (tdfxBlendFactor[state->src_blend] <<  8) |
+                                 (tdfxBlendFactor[state->src_blend] << 16) |
+                                 (tdfxBlendFactor[state->dst_blend] << 12) |
+                                 (tdfxBlendFactor[state->dst_blend] << 20);
 
      tdev->v_alphaMode = 1;
 }
@@ -381,12 +394,14 @@ static inline void tdfx_set_clip( TDFXDriverData *tdrv,
 
 /* required implementations */
 
-static void tdfxEngineSync( void *drv, void *dev )
+static DFBResult tdfxEngineSync( void *drv, void *dev )
 {
      TDFXDriverData *tdrv = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev = (TDFXDeviceData*) dev;
 
      tdfx_waitidle( tdrv, tdev );
+
+     return DFB_OK;
 }
 
 #define TDFX_SUPPORTED_DRAWINGFLAGS \
@@ -417,7 +432,7 @@ static void tdfxCheckState( void *drv, void *dev,
      /* if there are no other blitting flags than the supported
         and the source and destination formats are the same */
      if (!(state->blittingflags & ~TDFX_SUPPORTED_BLITTINGFLAGS)  &&
-         state->source  &&  state->source->format != DSPF_RGB24)
+         state->source  &&  state->source->config.format != DSPF_RGB24)
           state->accel |= TDFX_SUPPORTED_BLITTINGFUNCTIONS;
 }
 
@@ -428,22 +443,22 @@ static void tdfxSetState( void *drv, void *dev,
      TDFXDriverData *tdrv = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev = (TDFXDeviceData*) dev;
 
-     if (state->modified & SMF_DESTINATION)
+     if (state->mod_hw & SMF_DESTINATION)
           tdev->v_destination2D = tdev->v_destination3D = tdev->v_colorFore = 0;
 
-     if (state->modified & SMF_SOURCE)
+     if (state->mod_hw & SMF_SOURCE)
           tdev->v_source2D = 0;
 
-     if (state->modified & (SMF_DST_BLEND | SMF_SRC_BLEND))
+     if (state->mod_hw & (SMF_DST_BLEND | SMF_SRC_BLEND))
           tdev->v_alphaMode = 0;
 
-     if (state->modified & SMF_COLOR)
+     if (state->mod_hw & SMF_COLOR)
           tdev->v_color1 = tdev->v_colorFore = 0;
 
-     if (state->modified & SMF_SRC_COLORKEY)
+     if (state->mod_hw & SMF_SRC_COLORKEY)
           tdev->v_srcColorkey = 0;
 
-     if (state->modified & SMF_BLITTING_FLAGS)
+     if (state->mod_hw & SMF_BLITTING_FLAGS)
           tdev->v_commandExtra = 0;
 
      switch (accel) {
@@ -457,8 +472,7 @@ static void tdfxSetState( void *drv, void *dev,
 
                     funcs->FillRectangle = tdfxFillRectangle3D;
                     funcs->FillTriangle = tdfxFillTriangle3D;
-               }
-               else {
+               } else {
                     tdfx_validate_colorFore( tdrv, tdev, state );
                     tdfx_validate_destination2D( tdrv, tdev, state );
 
@@ -481,21 +495,18 @@ static void tdfxSetState( void *drv, void *dev,
                state->set |= DFXL_BLIT | DFXL_STRETCHBLIT;
                break;
 
-          case DFXL_DRAWSTRING:
-          case DFXL_ALL:
-               /* these are phony acceleration functions */
           default:
-               BUG( "unexpected drawing/blitting function" );
+               D_BUG( "unexpected drawing/blitting function" );
                break;
      }
 
-     if (state->modified & SMF_CLIP)
+     if (state->mod_hw & SMF_CLIP)
           tdfx_set_clip( tdrv, tdev, &state->clip );
 
-     state->modified = 0;
+     state->mod_hw = 0;
 }
 
-static void tdfxFillRectangle2D( void *drv, void *dev, DFBRectangle *rect )
+static bool tdfxFillRectangle2D( void *drv, void *dev, DFBRectangle *rect )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -507,9 +518,11 @@ static void tdfxFillRectangle2D( void *drv, void *dev, DFBRectangle *rect )
      voodoo2D->dstSize = ((rect->h & 0x1FFF) << 16) | (rect->w & 0x1FFF);
 
      voodoo2D->command = 5 | (1 << 8) | (0xCC << 24);
+
+     return true;
 }
 
-static void tdfxFillRectangle3D( void *drv, void *dev, DFBRectangle *rect )
+static bool tdfxFillRectangle3D( void *drv, void *dev, DFBRectangle *rect )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -533,13 +546,16 @@ static void tdfxFillRectangle3D( void *drv, void *dev, DFBRectangle *rect )
      voodoo3D->vertexBy = S12_4(rect->y);
 
      voodoo3D->triangleCMD = 0;
+
+     return true;
 }
 
-static void tdfxDrawRectangle( void *drv, void *dev, DFBRectangle *rect )
+static bool tdfxDrawRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
+     return false;
 }
 
-static void tdfxDrawLine2D( void *drv, void *dev, DFBRegion *line )
+static bool tdfxDrawLine2D( void *drv, void *dev, DFBRegion *line )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -550,9 +566,11 @@ static void tdfxDrawLine2D( void *drv, void *dev, DFBRegion *line )
      voodoo2D->srcXY   = ((line->y1 & 0x1FFF) << 16) | (line->x1 & 0x1FFF);
      voodoo2D->dstXY   = ((line->y2 & 0x1FFF) << 16) | (line->x2 & 0x1FFF);
      voodoo2D->command = 6 | (1 << 8) | (0xCC << 24);
+
+     return true;
 }
 
-/*static void tdfxDrawLine3D( void *drv, void *dev, DFBRegion *line )
+/*static bool tdfxDrawLine3D( void *drv, void *dev, DFBRegion *line )
 {
      int xl, xr, yb, yt;
 
@@ -592,9 +610,11 @@ static void tdfxDrawLine2D( void *drv, void *dev, DFBRegion *line )
      voodoo3D->vertexBy = S12_4_( line->y1, yt );
 
      voodoo3D->triangleCMD = 0;
+
+     return true;
 }*/
 
-static void tdfxFillTriangle2D( void *drv, void *dev, DFBTriangle *tri )
+static bool tdfxFillTriangle2D( void *drv, void *dev, DFBTriangle *tri )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -602,27 +622,27 @@ static void tdfxFillTriangle2D( void *drv, void *dev, DFBTriangle *tri )
 
      tdfx_waitfifo( tdrv, tdev, 7 );
 
-     sort_triangle( tri );
+     dfb_sort_triangle( tri );
 
      voodoo2D->srcXY = ((tri->y1 & 0x1FFF) << 16) | (tri->x1 & 0x1FFF);
      voodoo2D->command = 8 | (1 << 8) | (0xCC << 24);
 
      if (tri->x2 < tri->x3) {
-        voodoo2D->launchArea[0] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
-        voodoo2D->launchArea[1] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
-        voodoo2D->launchArea[2] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
+          voodoo2D->launchArea[0] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
+          voodoo2D->launchArea[1] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
+          voodoo2D->launchArea[2] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
+
+     } else {
+          voodoo2D->launchArea[0] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
+          voodoo2D->launchArea[1] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
+          voodoo2D->launchArea[2] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
 
      }
-     else {
-        voodoo2D->launchArea[0] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
-        voodoo2D->launchArea[1] = ((tri->y2 & 0x1FFF) << 16) | (tri->x2 & 0x1FFF);
-        voodoo2D->launchArea[2] = ((tri->y3 & 0x1FFF) << 16) | (tri->x3 & 0x1FFF);
-
-    }
 
+     return true;
 }
 
-static void tdfxFillTriangle3D( void *drv, void *dev, DFBTriangle *tri )
+static bool tdfxFillTriangle3D( void *drv, void *dev, DFBTriangle *tri )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -630,7 +650,7 @@ static void tdfxFillTriangle3D( void *drv, void *dev, DFBTriangle *tri )
 
      tdfx_waitfifo( tdrv, tdev, 7 );
 
-     sort_triangle( tri );
+     dfb_sort_triangle( tri );
 
      voodoo3D->vertexAx = S12_4(tri->x1);
      voodoo3D->vertexAy = S12_4(tri->y1);
@@ -644,15 +664,17 @@ static void tdfxFillTriangle3D( void *drv, void *dev, DFBTriangle *tri )
      voodoo3D->triangleCMD = (1 << 31);
 
      voodoo3D->triangleCMD = 0;
+
+     return true;
 }
 
-static void tdfxBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
+static bool tdfxBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
      Voodoo2D       *voodoo2D = tdrv->voodoo2D;
 
-     __u32 cmd = 1 | (1 <<8) | (0xCC << 24);//SST_2D_GO | SST_2D_SCRNTOSCRNBLIT | (ROP_COPY << 24);
+     u32 cmd = 1 | (1 <<8) | (0xCC << 24);//SST_2D_GO | SST_2D_SCRNTOSCRNBLIT | (ROP_COPY << 24);
 
      if (rect->x <= dx) {
           cmd |= (1 << 14);//SST_2D_X_RIGHT_TO_LEFT;
@@ -673,9 +695,11 @@ static void tdfxBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
      voodoo2D->dstSize = ((rect->h & 0x1FFF) << 16) | (rect->w & 0x1FFF);
 
      voodoo2D->command = cmd;
+
+     return true;
 }
 
-static void tdfxStretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
+static bool tdfxStretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
 {
      TDFXDriverData *tdrv     = (TDFXDriverData*) drv;
      TDFXDeviceData *tdev     = (TDFXDeviceData*) dev;
@@ -690,30 +714,25 @@ static void tdfxStretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangl
      voodoo2D->dstSize = ((dr->h & 0x1FFF) << 16) | (dr->w & 0x1FFF);
 
      voodoo2D->command = 2 | (1 << 8) | (0xCC << 24);
+
+     return true;
 }
 
 /* exported symbols */
 
-int
-driver_get_abi_version()
+static int
+driver_probe( CoreGraphicsDevice *device )
 {
-     return DFB_GRAPHICS_DRIVER_ABI_VERSION;
-}
-
-int
-driver_probe( GraphicsDevice *device )
-{
-#ifdef FB_ACCEL_3DFX_BANSHEE
-     switch (gfxcard_get_accelerator( device )) {
+     switch (dfb_gfxcard_get_accelerator( device )) {
           case FB_ACCEL_3DFX_BANSHEE:          /* Banshee/Voodoo3 */
                return 1;
      }
-#endif
+
      return 0;
 }
 
-void
-driver_get_info( GraphicsDevice     *device,
+static void
+driver_get_info( CoreGraphicsDevice *device,
                  GraphicsDriverInfo *info )
 {
      /* fill driver info structure */
@@ -723,7 +742,7 @@ driver_get_info( GraphicsDevice     *device,
 
      snprintf( info->vendor,
                DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 1;
@@ -732,14 +751,16 @@ driver_get_info( GraphicsDevice     *device,
      info->device_data_size = sizeof (TDFXDeviceData);
 }
 
-DFBResult
-driver_init_driver( GraphicsDevice      *device,
+static DFBResult
+driver_init_driver( CoreGraphicsDevice  *device,
                     GraphicsDeviceFuncs *funcs,
-                    void                *driver_data )
+                    void                *driver_data,
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      TDFXDriverData *tdrv = (TDFXDriverData*) driver_data;
 
-     tdrv->mmio_base = (volatile __u8*) gfxcard_map_mmio( device, 0, -1 );
+     tdrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!tdrv->mmio_base)
           return DFB_IO;
 
@@ -759,8 +780,8 @@ driver_init_driver( GraphicsDevice      *device,
 }
 
 
-DFBResult
-driver_init_device( GraphicsDevice     *device,
+static DFBResult
+driver_init_device( CoreGraphicsDevice *device,
                     GraphicsDeviceInfo *device_info,
                     void               *driver_data,
                     void               *device_data )
@@ -806,23 +827,19 @@ driver_init_device( GraphicsDevice     *device,
      voodoo2D->rop = 0xAAAAAA;
 
      tdfx_waitfifo( tdrv, tdev, 1 );           /* VOODOO !!!  */
-     *((__u32*)((__u8*) tdrv->mmio_base + 0x10c)) = 1 << 4 | 1 << 8 | 5 << 12 |
-                                                    1 << 18 | 5 << 24;
+
+     *((volatile u32*)((volatile u8*) tdrv->mmio_base + 0x10c)) =
+          1 << 4 | 1 << 8 | 5 << 12 | 1 << 18 | 5 << 24;
 
      dfb_config->pollvsync_after = 1;
 
      return DFB_OK;
 }
 
-void
-driver_init_layers()
-{
-}
-
-void
-driver_close_device( GraphicsDevice *device,
-                     void           *driver_data,
-                     void           *device_data )
+static void
+driver_close_device( CoreGraphicsDevice *device,
+                     void               *driver_data,
+                     void               *device_data )
 {
      TDFXDeviceData *tdev = (TDFXDeviceData*) device_data;
      TDFXDriverData *tdrv = (TDFXDriverData*) driver_data;
@@ -830,38 +847,38 @@ driver_close_device( GraphicsDevice *device,
      (void) tdev;
      (void) tdrv;
 
-     DEBUGMSG( "DirectFB/TDFX: FIFO Performance Monitoring:\n" );
-     DEBUGMSG( "DirectFB/TDFX:  %9d tdfx_waitfifo calls\n",
+     D_DEBUG( "DirectFB/TDFX: FIFO Performance Monitoring:\n" );
+     D_DEBUG( "DirectFB/TDFX:  %9d tdfx_waitfifo calls\n",
                tdev->waitfifo_calls );
-     DEBUGMSG( "DirectFB/TDFX:  %9d register writes (tdfx_waitfifo sum)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d register writes (tdfx_waitfifo sum)\n",
                tdev->waitfifo_sum );
-     DEBUGMSG( "DirectFB/TDFX:  %9d FIFO wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d FIFO wait cycles (depends on CPU)\n",
                tdev->fifo_waitcycles );
-     DEBUGMSG( "DirectFB/TDFX:  %9d IDLE wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d IDLE wait cycles (depends on CPU)\n",
                tdev->idle_waitcycles );
-     DEBUGMSG( "DirectFB/TDFX:  %9d FIFO space cache hits(depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d FIFO space cache hits(depends on CPU)\n",
                tdev->fifo_cache_hits );
-     DEBUGMSG( "DirectFB/TDFX: Conclusion:\n" );
-     DEBUGMSG( "DirectFB/TDFX:  Average register writes/tdfx_waitfifo"
+     D_DEBUG( "DirectFB/TDFX: Conclusion:\n" );
+     D_DEBUG( "DirectFB/TDFX:  Average register writes/tdfx_waitfifo"
                "call:%.2f\n",
                tdev->waitfifo_sum/(float)(tdev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/TDFX:  Average wait cycles/tdfx_waitfifo call:"
+     D_DEBUG( "DirectFB/TDFX:  Average wait cycles/tdfx_waitfifo call:"
                " %.2f\n",
                tdev->fifo_waitcycles/(float)(tdev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/TDFX:  Average fifo space cache hits: %02d%%\n",
+     D_DEBUG( "DirectFB/TDFX:  Average fifo space cache hits: %02d%%\n",
                (int)(100 * tdev->fifo_cache_hits/
-               (float)(tdev->waitfifo_calls)) );
+                     (float)(tdev->waitfifo_calls)) );
 
-     DEBUGMSG( "DirectFB/TDFX:  Pixels Out: %d\n", tdrv->voodoo3D->fbiPixelsOut );
-     DEBUGMSG( "DirectFB/TDFX:  Triangles Out: %d\n", tdrv->voodoo3D->fbiTrianglesOut );
+     D_DEBUG( "DirectFB/TDFX:  Pixels Out: %d\n", tdrv->voodoo3D->fbiPixelsOut );
+     D_DEBUG( "DirectFB/TDFX:  Triangles Out: %d\n", tdrv->voodoo3D->fbiTrianglesOut );
 }
 
-void
-driver_close_driver( GraphicsDevice *device,
-                     void           *driver_data )
+static void
+driver_close_driver( CoreGraphicsDevice *device,
+                     void               *driver_data )
 {
      TDFXDriverData *tdrv = (TDFXDriverData*) driver_data;
 
-     gfxcard_unmap_mmio( device, tdrv->mmio_base, -1 );
+     dfb_gfxcard_unmap_mmio( device, tdrv->mmio_base, -1 );
 }
 
