@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -116,6 +116,13 @@ dfb_pixel_to_color( DFBSurfacePixelFormat  format,
                ret_color->b = EXPAND_4to8( (pixel & 0x000f)       );
                break;
 
+          case DSPF_RGBA4444:
+               ret_color->r = EXPAND_4to8( (pixel         ) >> 12 );
+               ret_color->g = EXPAND_4to8( (pixel & 0x0f00) >>  8 );
+               ret_color->b = EXPAND_4to8( (pixel & 0x00f0) >>  4 );
+               ret_color->a = EXPAND_4to8( (pixel & 0x000f)       );
+               break;
+
           case DSPF_RGB16:
                ret_color->r = EXPAND_5to8( (pixel & 0xf800) >> 11 );
                ret_color->g = EXPAND_6to8( (pixel & 0x07e0) >>  5 );
@@ -170,6 +177,9 @@ dfb_pixel_from_color( DFBSurfacePixelFormat  format,
           case DSPF_ARGB4444:
                return PIXEL_ARGB4444( color->a, color->r, color->g, color->b );
 
+          case DSPF_RGBA4444:
+               return PIXEL_RGBA4444( color->a, color->r, color->g, color->b );
+
           case DSPF_RGB444:
                return PIXEL_RGB444( color->r, color->g, color->b );
 
@@ -236,7 +246,7 @@ dfb_pixelformat_name( DFBSurfacePixelFormat format )
 
 void
 dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
-                      void                  *src,
+                      const void            *src,
                       int                    spitch,
                       int                    surface_height,
                       u16                   *dst,
@@ -244,11 +254,8 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
                       int                    width,
                       int                    height )
 {
-     int  x;
-     int  dp2 = dpitch / 2;
-     u8  *src8;
-     u16 *src16;
-     u32 *src32;
+     const int dp2 = dpitch / 2;
+     int       x;
 
      switch (format) {
           case DSPF_RGB16:
@@ -262,8 +269,8 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
 
           case DSPF_NV16:
                while (height--) {
-                    src8  = src;
-                    src16 = src + surface_height * spitch;
+                    const u8  *src8  = src;
+                    const u16 *src16 = src + surface_height * spitch;
      
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -285,22 +292,36 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
           case DSPF_RGB444:
           case DSPF_ARGB4444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
-                         dst[x] = PIXEL_RGB16( ((src16[x] & 0x0f00) >> 4) | ((src16[x] & 0x0f00) >> 8),
-                                               ((src16[x] & 0x00f0)     ) | ((src16[x] & 0x00f0) >> 4),
-                                               ((src16[x] & 0x000f) << 4) | ((src16[x] & 0x000f)     ) );
+                         dst[x] = PIXEL_RGB16( EXPAND_4to8( (src16[x] & 0x0f00) >> 8 ),
+                                               EXPAND_4to8( (src16[x] & 0x00f0) >> 4 ),
+                                               EXPAND_4to8( (src16[x] & 0x000f)      ) );
 
                     src += spitch;
                     dst += dp2;
                }
                break;
 
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (x=0; x<width; x++)
+                         dst[x] = PIXEL_RGB16( EXPAND_4to8( (src16[x] & 0xf000) >> 12 ),
+                                               EXPAND_4to8( (src16[x] & 0x0f00) >>  8 ),
+                                               EXPAND_4to8( (src16[x] & 0x00f0) >>  4 ) );
+
+                    src += spitch;
+                    dst += dp2;
+               }
+               break;
+               
           case DSPF_RGB555:
           case DSPF_ARGB1555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = ((src16[x] & 0x7c00) << 1) | ((src16[x] & 0x03e0) << 1) | (src16[x] & 0x003f);
@@ -310,9 +331,9 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
                }
                break;
 
- 	  case DSPF_BGR555:
+          case DSPF_BGR555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = ((src16[x] & 0x7c00) >> 10) | ((src16[x] & 0x03e0) << 1) | ((src16[x] & 0x001f) << 11 );
@@ -325,7 +346,7 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
           case DSPF_RGB32:
           case DSPF_ARGB:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_RGB16( (src32[x] & 0xff0000) >> 16,
@@ -339,7 +360,7 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
 
           case DSPF_AYUV:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -361,7 +382,7 @@ dfb_convert_to_rgb16( DFBSurfacePixelFormat  format,
 
 void
 dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
-                       void                  *src,
+                       const void            *src,
                        int                    spitch,
                        int                    surface_height,
                        u16                   *dst,
@@ -369,11 +390,8 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
                        int                    width,
                        int                    height )
 {
-     int  x;
-     int  dp2 = dpitch / 2;
-     u8  *src8;
-     u16 *src16;
-     u32 *src32;
+     const int dp2 = dpitch / 2;
+     int       x;
 
      switch (format) {
           case DSPF_RGB555:
@@ -388,8 +406,8 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
 
           case DSPF_NV16:
                while (height--) {
-                    src8  = src;
-                    src16 = src + surface_height * spitch;
+                    const u8  *src8  = src;
+                    const u16 *src16 = src + surface_height * spitch;
      
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -411,12 +429,26 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
           case DSPF_RGB444:
           case DSPF_ARGB4444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
-                         dst[x] = PIXEL_RGB555( ((src16[x] & 0x0f00) >> 4) | ((src16[x] & 0x0f00) >> 8),
-                                                ((src16[x] & 0x00f0)     ) | ((src16[x] & 0x00f0) >> 4),
-                                                ((src16[x] & 0x000f) << 4) | ((src16[x] & 0x000f)     ) );
+                         dst[x] = PIXEL_RGB555( EXPAND_4to8( (src16[x] & 0x0f00) >> 8 ),
+                                                EXPAND_4to8( (src16[x] & 0x00f0) >> 4 ),
+                                                EXPAND_4to8( (src16[x] & 0x000f)      ) );
+
+                    src += spitch;
+                    dst += dp2;
+               }
+               break;
+
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (x=0; x<width; x++)
+                         dst[x] = PIXEL_RGB555( EXPAND_4to8( (src16[x] & 0xf000) >> 12 ),
+                                                EXPAND_4to8( (src16[x] & 0x0f00) >>  8 ),
+                                                EXPAND_4to8( (src16[x] & 0x00f0) >>  4 ) );
 
                     src += spitch;
                     dst += dp2;
@@ -425,7 +457,7 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB16:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = ((src16[x] & 0xffc0) >> 1) | (src16[x] & 0x001f);
@@ -437,7 +469,7 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
 
  	  case DSPF_BGR555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = ((src16[x] & 0x7c00) >> 10) | (src16[x] & 0x03e0) | ((src16[x] & 0x001f) << 10 );
@@ -450,7 +482,7 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
           case DSPF_RGB32:
           case DSPF_ARGB:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_RGB555( (src32[x] & 0xff0000) >> 16,
@@ -464,7 +496,7 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
 
           case DSPF_AYUV:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -486,7 +518,7 @@ dfb_convert_to_rgb555( DFBSurfacePixelFormat  format,
 
 void
 dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
-                      void                  *src,
+                      const void            *src,
                       int                    spitch,
                       int                    surface_height,
                       u32                   *dst,
@@ -494,11 +526,8 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
                       int                    width,
                       int                    height )
 {
-     int  x;
-     int  dp4 = dpitch / 4;
-     u8  *src8;
-     u16 *src16;
-     u32 *src32;
+     const int dp4 = dpitch / 4;
+     int       x;
 
      switch (format) {
           case DSPF_RGB32:
@@ -511,9 +540,29 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
                }
                break;
 
+          case DSPF_RGB24:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    for (x=0; x<width; x++)
+#ifdef WORDS_BIGENDIAN
+                         dst[x] = ( src8[x*3+0] << 16 ) |
+                                  ( src8[x*3+1] <<  8 ) |
+                                  ( src8[x*3+2]       );
+#else
+                         dst[x] = ( src8[x*3+2] << 16 ) |
+                                  ( src8[x*3+1] <<  8 ) |
+                                  ( src8[x*3+0]       );
+#endif
+
+                    src += spitch;
+                    dst += dp4;
+               }
+               break;
+
           case DSPF_AYUV:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -530,8 +579,8 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
 
           case DSPF_NV16:
                while (height--) {
-                    src8  = src;
-                    src16 = src + surface_height * spitch;
+                    const u8  *src8  = src;
+                    const u16 *src16 = src + surface_height * spitch;
      
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -553,12 +602,22 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
           case DSPF_RGB444:
           case DSPF_ARGB4444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
-                         dst[x] = PIXEL_RGB32( ((src16[x] & 0x0f00) >> 4) | ((src16[x] & 0x0f00) >> 8),
-                                               ((src16[x] & 0x00f0)     ) | ((src16[x] & 0x00f0) >> 4),
-                                               ((src16[x] & 0x000f) << 4) | ((src16[x] & 0x000f)     ) );
+                         dst[x] = ARGB4444_TO_RGB32( src16[x] );
+
+                    src += spitch;
+                    dst += dp4;
+               }
+               break;
+
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (x=0; x<width; x++)
+                         dst[x] = RGBA4444_TO_RGB32( src16[x] );
 
                     src += spitch;
                     dst += dp4;
@@ -568,7 +627,7 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
           case DSPF_RGB555:
           case DSPF_ARGB1555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_RGB32( ((src16[x] & 0x7c00) >> 7) | ((src16[x] & 0x7000) >> 12),
@@ -582,7 +641,7 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
 
           case DSPF_BGR555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_RGB32( ((src16[x] & 0x001f) << 3) | ((src16[x] & 0x001c) >> 2),
@@ -596,7 +655,7 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB16:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_RGB32( ((src16[x] & 0xf800) >> 8) | ((src16[x] & 0xe000) >> 13),
@@ -615,7 +674,7 @@ dfb_convert_to_rgb32( DFBSurfacePixelFormat  format,
 
 void
 dfb_convert_to_argb( DFBSurfacePixelFormat  format,
-                     void                  *src,
+                     const void            *src,
                      int                    spitch,
                      int                    surface_height,
                      u32                   *dst,
@@ -623,11 +682,8 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
                      int                    width,
                      int                    height )
 {
-     int  x;
-     int  dp4 = dpitch / 4;
-     u8  *src8;
-     u16 *src16;
-     u32 *src32;
+     const int dp4 = dpitch / 4;
+     int       x;
 
      switch (format) {
           case DSPF_ARGB:
@@ -641,7 +697,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB32:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = src32[x] | 0xff000000;
@@ -651,9 +707,29 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
                }
                break;
 
+          case DSPF_RGB24:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    for (x=0; x<width; x++)
+#ifdef WORDS_BIGENDIAN
+                         dst[x] = ( src8[x*3+0] << 16 ) |
+                                  ( src8[x*3+1] <<  8 ) |
+                                  ( src8[x*3+2]       ) | 0xff000000;
+#else
+                         dst[x] = ( src8[x*3+2] << 16 ) |
+                                  ( src8[x*3+1] <<  8 ) |
+                                  ( src8[x*3+0]       ) | 0xff000000;
+#endif
+
+                    src += spitch;
+                    dst += dp4;
+               }
+               break;
+
           case DSPF_AYUV:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
      
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -670,8 +746,8 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_NV16:
                while (height--) {
-                    src8  = src;
-                    src16 = src + surface_height * spitch;
+                    const u8  *src8  = src;
+                    const u16 *src16 = src + surface_height * spitch;
 
                     for (x=0; x<width; x++) {
                          int r, g, b;
@@ -692,13 +768,22 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_ARGB4444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
-                         dst[x] = PIXEL_ARGB( ((src16[x] & 0xf000) >> 8) | ((src16[x] & 0xf000) >> 12),
-                                              ((src16[x] & 0x0f00) >> 4) | ((src16[x] & 0x0f00) >> 8),
-                                              ((src16[x] & 0x00f0)     ) | ((src16[x] & 0x00f0) >> 4),
-                                              ((src16[x] & 0x000f) << 4) | ((src16[x] & 0x000f)     ) );
+                         dst[x] = ARGB4444_TO_ARGB( src16[x] );
+
+                    src += spitch;
+                    dst += dp4;
+               }
+               break;
+
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (x=0; x<width; x++)
+                         dst[x] = RGBA4444_TO_ARGB( src16[x] );
 
                     src += spitch;
                     dst += dp4;
@@ -707,7 +792,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_ARGB( 0xff,
@@ -722,7 +807,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_ARGB1555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_ARGB( (src16[x] & 0x8000) ? 0xff : 0x00,
@@ -737,7 +822,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_ARGB( 0xff,
@@ -752,7 +837,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_BGR555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_ARGB( 0xff,
@@ -767,7 +852,7 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 
           case DSPF_RGB16:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0; x<width; x++)
                          dst[x] = PIXEL_ARGB( 0xff,
@@ -786,8 +871,296 @@ dfb_convert_to_argb( DFBSurfacePixelFormat  format,
 }
 
 void
-dfb_convert_to_a4( DFBSurfacePixelFormat  format,
-                   void                  *src,
+dfb_convert_to_rgb24( DFBSurfacePixelFormat  format,
+                      const void            *src,
+                      int                    spitch,
+                      int                    surface_height,
+                      u8                    *dst,
+                      int                    dpitch,
+                      int                    width,
+                      int                    height )
+{
+     int n, n3;
+
+     switch (format) {
+          case DSPF_A8:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = src8[n];
+                         dst[n3+1] = src8[n];
+                         dst[n3+2] = src8[n];
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_AiRGB:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src32[n] & 0xFF0000) >> 16;
+                         dst[n3+1] = (src32[n] & 0x00FF00) >>  8;
+                         dst[n3+2] = (src32[n] & 0x0000FF);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src32[n] & 0xFF0000) >> 16;
+                         dst[n3+1] = (src32[n] & 0x00FF00) >>  8;
+                         dst[n3+2] = (src32[n] & 0x0000FF);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB1555:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0x7C00) >> 7;
+                         dst[n3+1] = (src16[n] & 0x03E0) >> 2;
+                         dst[n3+2] = (src16[n] & 0x001F) << 3;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB555:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0x7C00) >> 7;
+                         dst[n3+1] = (src16[n] & 0x03E0) >> 2;
+                         dst[n3+2] = (src16[n] & 0x001F) << 3;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+
+          case DSPF_BGR555:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+2] = (src16[n] & 0x7C00) >> 7;
+                         dst[n3+1] = (src16[n] & 0x03E0) >> 2;
+                         dst[n3+0] = (src16[n] & 0x001F) << 3;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+
+          case DSPF_ARGB2554:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0x3E00) >> 6;
+                         dst[n3+1] = (src16[n] & 0x01F0) >> 1;
+                         dst[n3+2] = (src16[n] & 0x000F) << 4;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0x0F00) >> 4;
+                         dst[n3+1] = (src16[n] & 0x00F0);
+                         dst[n3+2] = (src16[n] & 0x000F) << 4;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0xF000) >> 8;
+                         dst[n3+1] = (src16[n] & 0x0F00) >> 4;
+                         dst[n3+2] = (src16[n] & 0x00F0);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0x0F00) >> 4;
+                         dst[n3+1] = (src16[n] & 0x00F0);
+                         dst[n3+2] = (src16[n] & 0x000F) << 4;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB332:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = lookup3to8[ (src8[n] >> 5)        ];
+                         dst[n3+1] = lookup3to8[ (src8[n] >> 2) & 0x07 ];
+                         dst[n3+2] = lookup2to8[ (src8[n]     ) & 0x03 ];
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB16:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src16[n] & 0xF800) >> 8;
+                         dst[n3+1] = (src16[n] & 0x07E0) >> 3;
+                         dst[n3+2] = (src16[n] & 0x001F) << 3;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB24:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+#ifdef WORDS_BIGENDIAN
+                         dst[n3+0] = src8[n3+0];
+                         dst[n3+1] = src8[n3+1];
+                         dst[n3+2] = src8[n3+2];
+#else
+                         dst[n3+0] = src8[n3+2];
+                         dst[n3+1] = src8[n3+1];
+                         dst[n3+2] = src8[n3+0];
+#endif
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGB32:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0, n3=0; n<width; n++, n3+=3) {
+                         dst[n3+0] = (src32[n] & 0xFF0000) >> 16;
+                         dst[n3+1] = (src32[n] & 0x00FF00) >>  8;
+                         dst[n3+2] = (src32[n] & 0x0000FF);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_YUY2:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0, n3=0; n<width/2; n++, n3+=6) {
+                         register u32 y0, cb, y1, cr;
+                         y0 = (src32[n] & 0x000000FF);
+                         cb = (src32[n] & 0x0000FF00) >>  8;
+                         y1 = (src32[n] & 0x00FF0000) >> 16;
+                         cr = (src32[n] & 0xFF000000) >> 24;
+                         YCBCR_TO_RGB( y0, cb, cr,
+                                       dst[n3+0], dst[n3+1], dst[n3+2] );
+                         YCBCR_TO_RGB( y1, cb, cr,
+                                       dst[n3+3], dst[n3+4], dst[n3+5] );
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_UYVY:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0, n3=0; n<width/2; n++, n3+=6) {
+                         register u32 y0, cb, y1, cr;
+                         cb = (src32[n] & 0x000000FF);
+                         y0 = (src32[n] & 0x0000FF00) >>  8;
+                         cr = (src32[n] & 0x00FF0000) >> 16;
+                         y1 = (src32[n] & 0xFF000000) >> 24;
+                         YCBCR_TO_RGB( y0, cb, cr,
+                                       dst[n3+0], dst[n3+1], dst[n3+2] );
+                         YCBCR_TO_RGB( y1, cb, cr,
+                                       dst[n3+3], dst[n3+4], dst[n3+5] );
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_NV16: {
+               while (height--) {
+                    const u16 *cbcr = src + surface_height * spitch;
+                    const u8  *src8 = src;
+
+                    for (n=0, n3=0; n<width/2; n++, n3+=6) {
+#ifdef WORDS_BIGENDIAN
+                         YCBCR_TO_RGB( src8[n*2+0], cbcr[n] >> 8, cbcr[n] & 0xff,
+                                       dst[n3+0], dst[n3+1], dst[n3+2] );
+     
+                         YCBCR_TO_RGB( src8[n*2+1], cbcr[n] >> 8, cbcr[n] & 0xff,
+                                       dst[n3+3], dst[n3+4], dst[n3+5] );
+#else
+                         YCBCR_TO_RGB( src8[n*2+0], cbcr[n] & 0xff, cbcr[n] >> 8,
+                                       dst[n3+0], dst[n3+1], dst[n3+2] );
+     
+                         YCBCR_TO_RGB( src8[n*2+1], cbcr[n] & 0xff, cbcr[n] >> 8,
+                                       dst[n3+3], dst[n3+4], dst[n3+5] );
+#endif
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          }
+          default:
+               D_ONCE( "unsupported format" );
+     }
+}
+
+void
+dfb_convert_to_a8( DFBSurfacePixelFormat  format,
+                   const void            *src,
                    int                    spitch,
                    int                    surface_height,
                    u8                    *dst,
@@ -795,18 +1168,146 @@ dfb_convert_to_a4( DFBSurfacePixelFormat  format,
                    int                    width,
                    int                    height )
 {
-     int  x, n;
-     int  w2 = width / 2;
-     u8  *src8;
-     u16 *src16;
-     u32 *src32;
+     int n;
+
+     switch (format) {
+          case DSPF_A8:
+               while (height--) {
+                    const u8 *src8 = src;
+
+                    direct_memcpy( dst, src8, width );
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_AiRGB:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0; n<width; n++) {
+                         dst[n] = ~(src32[n] >> 24);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB:
+               while (height--) {
+                    const u32 *src32 = src;
+
+                    for (n=0; n<width; n++) {
+                         dst[n] = src32[n] >> 24;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB1555:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0; n<width; n++) {
+                         dst[n] = (src16[n] & 0x8000) ? 0xff : 0x00;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB2554:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0; n<width; n++) {
+                         switch (src16[n] >> 14) {
+                              case 0:
+                                   dst[n] = 0x00;
+                                   break;
+                              case 1:
+                                   dst[n] = 0x55;
+                                   break;
+                              case 2:
+                                   dst[n] = 0xAA;
+                                   break;
+                              case 3:
+                                   dst[n] = 0xFF;
+                                   break;
+                         }
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_ARGB4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0; n<width; n++) {
+                         dst[n]  = (src16[n] >> 12);
+                         dst[n] |= dst[n] << 4;
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (n=0; n<width; n++) {
+                         dst[n] = EXPAND_4to8(src16[n] & 0xf);
+                    }
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+
+          case DSPF_RGB332:
+          case DSPF_RGB444:
+          case DSPF_RGB555:
+          case DSPF_BGR555:
+          case DSPF_RGB16:
+          case DSPF_RGB24:
+          case DSPF_RGB32:
+          case DSPF_YUY2:
+          case DSPF_UYVY:
+          case DSPF_NV16:
+               while (height--) {
+                    memset( dst, 0xff, width );
+
+                    dst += dpitch;
+               }
+               break;
+          default:
+               D_ONCE( "unsupported format" );
+     }
+}
+
+void
+dfb_convert_to_a4( DFBSurfacePixelFormat  format,
+                   const void            *src,
+                   int                    spitch,
+                   int                    surface_height,
+                   u8                    *dst,
+                   int                    dpitch,
+                   int                    width,
+                   int                    height )
+{
+     const int w2 = width / 2;
+     int       x, n;
 
      D_ASSUME( (width & 1) == 0 );
 
      switch (format) {
           case DSPF_A8:
                while (height--) {
-                    src8 = src;
+                    const u8 *src8 = src;
 
                     for (x=0, n=0; x<w2; x++, n+=2)
                          dst[x] = (src8[n] & 0xf0) | ((src8[n+1] & 0xf0) >> 4);
@@ -818,7 +1319,7 @@ dfb_convert_to_a4( DFBSurfacePixelFormat  format,
 
           case DSPF_ARGB4444:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0, n=0; x<w2; x++, n+=2)
                          dst[x] = ((src16[n] & 0xf000) >> 8) | (src16[n+1] >> 12);
@@ -828,9 +1329,21 @@ dfb_convert_to_a4( DFBSurfacePixelFormat  format,
                }
                break;
 
+          case DSPF_RGBA4444:
+               while (height--) {
+                    const u16 *src16 = src;
+
+                    for (x=0, n=0; x<w2; x++, n+=2)
+                         dst[x] = ((src16[n] & 0x000f) << 4) | (src16[n+1] & 0x000f);
+
+                    src += spitch;
+                    dst += dpitch;
+               }
+               break;
+
           case DSPF_ARGB1555:
                while (height--) {
-                    src16 = src;
+                    const u16 *src16 = src;
 
                     for (x=0, n=0; x<w2; x++, n+=2)
                          dst[x] = ((src16[n] & 0x8000) ? 0xf0 : 0) | ((src16[n+1] & 0x8000) ? 0x0f : 0);
@@ -842,7 +1355,7 @@ dfb_convert_to_a4( DFBSurfacePixelFormat  format,
 
           case DSPF_ARGB:
                while (height--) {
-                    src32 = src;
+                    const u32 *src32 = src;
 
                     for (x=0, n=0; x<w2; x++, n+=2)
                          dst[x] = ((src32[n] & 0xf0000000) >> 24) | ((src32[n+1] & 0xf0000000) >> 28);
@@ -860,7 +1373,7 @@ dfb_convert_to_a4( DFBSurfacePixelFormat  format,
 
 void
 dfb_convert_to_yuy2( DFBSurfacePixelFormat  format,
-                     void                  *src,
+                     const void            *src,
                      int                    spitch,
                      int                    surface_height,
                      u32                   *dst,
@@ -868,7 +1381,7 @@ dfb_convert_to_yuy2( DFBSurfacePixelFormat  format,
                      int                    width,
                      int                    height )
 {
-     int dp4 = dpitch / 4;
+     const int dp4 = dpitch / 4;
 
      switch (format) {
           case DSPF_YUY2:
@@ -887,7 +1400,7 @@ dfb_convert_to_yuy2( DFBSurfacePixelFormat  format,
 
 void
 dfb_convert_to_uyvy( DFBSurfacePixelFormat  format,
-                     void                  *src,
+                     const void            *src,
                      int                    spitch,
                      int                    surface_height,
                      u32                   *dst,
@@ -895,7 +1408,7 @@ dfb_convert_to_uyvy( DFBSurfacePixelFormat  format,
                      int                    width,
                      int                    height )
 {
-     int dp4 = dpitch / 4;
+     const int dp4 = dpitch / 4;
 
      switch (format) {
           case DSPF_UYVY:
