@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -161,7 +161,7 @@ static bool input_filter_local( DFBEvent *evt,
 static bool input_filter_global( DFBEvent *evt,
                                  void     *ctx );
 
-static void drop_window( IDirectFB_data *data );
+static void drop_window( IDirectFB_data *data, bool enable_cursor );
 
 /**********************************************************************************************************************/
 
@@ -179,7 +179,7 @@ IDirectFB_Destruct( IDirectFB *thiz )
 
      D_DEBUG_AT( IDFB, "%s( %p )\n", __FUNCTION__, thiz );
 
-     drop_window( data );
+     drop_window( data, false );
 
      if (data->primary.context)
           dfb_layer_context_unref( data->primary.context );
@@ -271,7 +271,7 @@ IDirectFB_SetCooperativeLevel( IDirectFB           *thiz,
                          return ret;
                     }
 
-                    drop_window( data );
+                    drop_window( data, true );
 
                     data->primary.context = context;
                }
@@ -431,9 +431,9 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
      int height = 256;
      DFBSurfacePixelFormat format;
      DFBSurfaceCapabilities caps = DSCAPS_NONE;
-     DFBDisplayLayerConfig  config;
      CoreSurface *surface = NULL;
      unsigned long resource_id = 0;
+     DFBDisplayLayerConfig  config;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB)
 
@@ -444,7 +444,14 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
      else
           dfb_layer_context_get_configuration( data->context, &config );
 
-     format = config.pixelformat;
+     if (desc->flags & DSDESC_HINTS && desc->hints & DSHF_FONT) {
+          format = dfb_config->font_format;
+
+          if (dfb_config->font_premult)
+               caps = DSCAPS_PREMULTIPLIED;
+     }
+     else
+          format = config.pixelformat;
 
      if (!desc || !interface)
           return DFB_INVARG;
@@ -496,6 +503,7 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
           case DSPF_ARGB6666:
           case DSPF_ARGB2554:
           case DSPF_ARGB4444:
+          case DSPF_RGBA4444:
           case DSPF_AYUV:
           case DSPF_AiRGB:
           case DSPF_I420:
@@ -621,6 +629,7 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
 
                          switch (format) {
                               case DSPF_ARGB4444:
+                              case DSPF_RGBA4444:
                               case DSPF_ARGB2554:
                               case DSPF_ARGB1555:
                               case DSPF_ARGB:
@@ -640,7 +649,7 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
                          if (ret)
                               return ret;
 
-                         drop_window( data );
+                         drop_window( data, true );
 
                          data->primary.window = window;
 
@@ -1934,7 +1943,7 @@ input_filter_global( DFBEvent *evt,
 }
 
 static void
-drop_window( IDirectFB_data *data )
+drop_window( IDirectFB_data *data, bool enable_cursor )
 {
      if (!data->primary.window)
           return;
@@ -1945,6 +1954,6 @@ drop_window( IDirectFB_data *data )
      data->primary.window  = NULL;
      data->primary.focused = false;
 
-     dfb_windowstack_cursor_enable( data->core, data->stack, true );
+     dfb_windowstack_cursor_enable( data->core, data->stack, enable_cursor );
 }
 
