@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -59,12 +59,17 @@
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef USE_GLX
+#include <GL/glx.h>
+#include "glx_surface_pool.h"
+#endif
+
 #include "xwindow.h"
 #include "x11.h"
 #include "primary.h"
 
 
-D_DEBUG_DOMAIN( X11_Window, "X11/Window", "X11 Window" );
+D_DEBUG_DOMAIN( X11_Layer,  "X11/Layer",  "X11 Layer" );
 D_DEBUG_DOMAIN( X11_Update, "X11/Update", "X11 Update" );
 
 /**********************************************************************************************************************/
@@ -142,6 +147,8 @@ primaryInitScreen( CoreScreen           *screen,
                    void                 *screen_data,
                    DFBScreenDescription *description )
 {
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      /* Set the screen capabilities. */
      description->caps = DSCCAPS_NONE;
 
@@ -162,6 +169,8 @@ primaryGetScreenSize( CoreScreen *screen,
      DFBX11       *x11    = driver_data;
      DFBX11Shared *shared = x11->shared;
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      *ret_width  = shared->screen_size.w;
      *ret_height = shared->screen_size.h;
 
@@ -198,6 +207,8 @@ primaryInitLayer( CoreLayer                  *layer,
      DFBX11       *x11    = driver_data;
      DFBX11Shared *shared = x11->shared;
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      /* set capabilities and type */
      description->caps = DLCAPS_SURFACE;
      description->type = DLTF_GRAPHICS;
@@ -259,6 +270,8 @@ primaryTestRegion( CoreLayer                  *layer,
 {
      CoreLayerRegionConfigFlags fail = 0;
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      switch (config->buffermode) {
           case DLBM_FRONTONLY:
           case DLBM_BACKSYSTEM:
@@ -276,9 +289,11 @@ primaryTestRegion( CoreLayer                  *layer,
           case DSPF_NV16:
           case DSPF_RGB444:
           case DSPF_ARGB4444:
+          case DSPF_RGBA4444:
           case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_BGR555:
+          case DSPF_RGB24:
           case DSPF_RGB32:
           case DSPF_ARGB:
           case DSPF_AYUV:
@@ -325,6 +340,8 @@ primarySetRegion( CoreLayer                  *layer,
      DFBResult  ret;
      DFBX11    *x11 = driver_data;
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      ret = dfb_x11_create_window( x11, config );
      if (ret)
           return ret;
@@ -343,6 +360,8 @@ primaryRemoveRegion( CoreLayer             *layer,
 {
      DFBX11 *x11 = driver_data;
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      dfb_x11_destroy_window( x11 );
 
      return DFB_OK;
@@ -360,6 +379,8 @@ primaryFlipRegion( CoreLayer             *layer,
      DFBX11    *x11    = driver_data;
      DFBRegion  region = DFB_REGION_INIT_FROM_DIMENSION( &surface->config.size );
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      dfb_surface_flip( surface, false );
 
      return dfb_x11_update_screen( x11, &region, lock );
@@ -377,6 +398,8 @@ primaryUpdateRegion( CoreLayer             *layer,
      DFBX11    *x11    = driver_data;
      DFBRegion  region = DFB_REGION_INIT_FROM_DIMENSION( &surface->config.size );
 
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
+
      if (update && !dfb_region_region_intersect( &region, update ))
           return DFB_OK;
 
@@ -433,7 +456,7 @@ update_screen( DFBX11 *x11, const DFBRectangle *clip, CoreSurfaceBufferLock *loc
      allocation = lock->allocation;
      CORE_SURFACE_ALLOCATION_ASSERT( allocation );
 
-     surface = allocation->buffer->surface;
+     surface = allocation->surface;
      D_ASSERT( surface != NULL );
 
 
@@ -448,6 +471,33 @@ update_screen( DFBX11 *x11, const DFBRectangle *clip, CoreSurfaceBufferLock *loc
 
      D_DEBUG_AT( X11_Update, "  -> %4d,%4d-%4dx%4d\n", DFB_RECTANGLE_VALS( &rect ) );
 
+#ifdef USE_GLX
+     /* Check for GLX allocation... */
+     if (allocation->pool == shared->glx_pool && lock->handle) {
+          LocalPixmap *pixmap = lock->handle;
+
+          D_MAGIC_ASSERT( pixmap, LocalPixmap );
+
+          /* ...and just call SwapBuffers... */
+          //D_DEBUG_AT( X11_Update, "  -> Calling glXSwapBuffers( 0x%lx )...\n", alloc->drawable );
+          //glXSwapBuffers( x11->display, alloc->drawable );
+
+
+          D_DEBUG_AT( X11_Update, "  -> Copying from GLXPixmap...\n" );
+
+          glXWaitGL();
+
+          XCopyArea( x11->display, pixmap->pixmap, xw->window, xw->gc,
+                     rect.x, rect.y, rect.w, rect.h, rect.x, rect.y );
+
+          glXWaitX();
+
+          XUnlockDisplay( x11->display );
+
+          return DFB_OK;
+     }
+#endif
+
      /* Check for our special native allocation... */
      if (allocation->pool == shared->x11image_pool && lock->handle) {
           x11Image *image = lock->handle;
@@ -532,9 +582,9 @@ dfb_x11_create_window_handler( DFBX11 *x11, CoreLayerRegionConfig *config )
      XWindow      *xw;
      DFBX11Shared *shared = x11->shared;
 
-     D_DEBUG_AT( X11_Window, "%s( %p )\n", __FUNCTION__, config );
+     D_DEBUG_AT( X11_Layer, "%s( %p )\n", __FUNCTION__, config );
 
-     D_DEBUG_AT( X11_Window, "  -> %4dx%4d %s\n", config->width, config->height, dfb_pixelformat_name(config->format) );
+     D_DEBUG_AT( X11_Layer, "  -> %4dx%4d %s\n", config->width, config->height, dfb_pixelformat_name(config->format) );
 
      XLockDisplay( x11->display );
 
@@ -570,7 +620,7 @@ dfb_x11_destroy_window_handler( DFBX11 *x11 )
 {
      DFBX11Shared *shared = x11->shared;
 
-     D_DEBUG_AT( X11_Window, "%s()\n", __FUNCTION__ );
+     D_DEBUG_AT( X11_Layer, "%s()\n", __FUNCTION__ );
 
      XLockDisplay( x11->display );
 
